package DBIx::Interpolate;

our $VERSION = '0.29';

#IMRPOVE: provide a Tie to access DBI attributes?  e.g.
# $db->{mysql_insert_id}?

use strict;
use warnings;
use DBI;
use Carp;
use SQL::Interpolate qw(:all);

sub AUTOLOAD
{
    my($self, @args) = @_;
    (my $meth = our $AUTOLOAD) =~ s/.*:://;
    #print "[$meth]";
    return unless $meth =~ /[^A-Z]/; # e.g. DESTROY

    my @x;
    my $want = wantarray();
    my $dbh = $self->{dbh};

    eval {
        my @new_args;

        if($meth =~ /^( do | select.* | prepare.* )$/x) {
            @new_args = $self->{interp}->dbi_interp(@args);
        }
        else {
            @new_args = @args;
        }
        # Is there a simpler way?
        if($want)             { @x    = $dbh->$meth(@new_args); }
        elsif(defined($want)) { $x[0] = $dbh->$meth(@new_args); }
        else                  {         $dbh->$meth(@new_args); }
    };
    if($@) { croak $@; }

    return @x    if $want;
    return $x[0] if defined($want);
    return;
}

sub new
{
    my($class, $db, %params) = @_;

    my $own_dbh = 0;
    if(ref($db) eq 'ARRAY') {
        $db = DBI->connect(@$db)
    }
    my $interp = new SQL::Interpolate(dbh => $db);

    my $self = bless {
        dbh    => $db,
        interp => $interp
    }, $class;
    return $self;
}

sub connect
{
    my $class = shift;
    my $self;
    eval {
        my $dbh = DBI->connect(@_);
        return if !$dbh;
        $self = new DBIx::Interpolate($dbh);  #Q: OK?
    };
    if($@) { croak $@; }
    return $self;
}

sub dbh
{
    my $self = shift;
    return $self->{dbh};
}

sub interp
{
    my $self = shift;
    return $self->{interp};
}

1

__END__

=head1 NAME

DBIx::Interpolate - Wrapper for DBI to interpolate variables into SQL

=head1 SYNOPSIS

  use DBIx::Interpolate;
  use SQL::Interpolate FILTER => 1, qw(:all);
  
  my $dbx = DBIx::Interpolate->connect(
      $data_source, $username, $auth, \%attr);
  my $rows = $dbx->selectall_arrayref(sql[
      SELECT * FROM table
      WHERE color IN @colors
            AND y = $x
      LIMIT(start => 1, count => 10)
  ]);

=head1 DESCRIPTION

This module is closely related to L<SQL::Interpolate|SQL::Interpolate>,
and fuller description of this module is provided there.

DBIx::Interpolate is a wrapper around L<DBI|DBI> and does nothing more
than bring SQL::Interpolate behavior to DBI.  The DBIx::Interpolate
interface is very close to that of DBI.  All methods look and behave
identically to their DBI counterpart except that certain methods, such
as the do, select.*, and prepare.* methods, expect a list of input
parameters in the format expected by
L<SQL::Interpolate::dbi_interp()|SQL::Interpolate/item_dbi_interp>
rather than the typical ($statement, \%attr, @bind_values) of DBI.  In
fact, internally, the parameters for DBIx::Interpolate methods are
passed to SQL::Interpolate::dbi_interp(), and the result is then
passed onto the respective DBI method.  Therefore, DBIx::Interpolate
is quite thin and follows a design goal of maintaining as much
compatibility with DBI as possible.

DBIx::Interpolate allows you to write

  $dbx->fetchall_arrayref(qq[
      SELECT * from mytable
      WHERE height > ], \$x
  );

rather than using the dbi_interp "adaptor" function as in

  $dbh->fetchall_arrayref(dbi_interp $dbh, qq[
      SELECT * from mytable
      WHERE height > ], \$x
  );

Internally, this module will pass the database handle to
the SQL::Interpolate so that the generated SQL can be specific
to the SQL dialect of your database (e.g. LIMIT clause syntax).

You might also wish to use this module with source filtering
enabled on the SQL::Interpolate module in order to instead use
the sql// quote-like operator:

  $dbx->fetchall_arrayref(sql[
      SELECT * from mytable
      WHERE height > $x
  ]);

=head2 METHODS

=over 4

=item C<new>

Creates a new object and optionally creates or attached a DBI handle.

 my $dbx = new DBX::Interpolate($db, %params);

$db [optional] is either a DBI database handle or an ARRAYREF
containing parameters to pass to DBI::connect, e.g.
[$data_source, $username, $auth, \%attr].

%params is currently ignored. FIX? accept SQL::Interpolate params?

=item C<connect>

Connects to a database.

 $dbx = DBIx::Interpolate->connect($data_source, $username, $auth, \%attr);

This is identical to DBI::Connect except that it returns at
DBIx::Interpolate object.  An alternate way to connect or attach an
existing DBI handle is via the C<new> method.

=item C<dbh>

 $dbh = $dbx->dbh();

Returns the underlying DBI handle $dbh.  The is useful if you need to
pass the DBI handle to code that does not use SQL::Interpolate.

 $dbx->dbh()->selectall_arrayref(
     "SELECT * FROM mytable WHERE x = ?", undef, $x);

=item C<interp>

 $interp = $dbx->interp();

Returns the underlying SQL::Interpolate object $interp.  This is useful
if you need to configure $interp.

 $dbx->interp()->relations(...);

=item do | select.* | prepare.*

These methods are identical to those in DBI except that it takes a parameter
list identical to SQL::Interpolate::dbi_interp().

 my $res = $dbx->selectall_hashref(sql[SELECT * FROM mytable WHERE x=$x]);

=back

=head1 CREDITS

This module and its design were originally suggested by Jim Cromie.

=head1 SEE ALSO

L<SQL::Interpolate|SQL::Interpolate>,
L<DBI|DBI>,
L<SQL::Abstract|SQL::Abstract>,
L<DBIx::Abstract|DBIx::Abstract>,
L<Class::DBI|Class::DBI>.

=head1 AUTHOR

David Manura (http://math2.org/david).

Bug reports and comments on the design are most welcome.

=head1 COPYRIGHT

  Copyright (c) 2004-2005, David Manura. All Rights Reserved.
  This module is free software. It may be used, redistributed
  and/or modified under the same terms as Perl itself.

=cut
