package SQL::Interpolate::Filter;

use strict;
use warnings;
use Filter::Simple;
use Text::Balanced qw/extract_quotelike
                      extract_bracketed
                      extract_multiple
                      extract_variable
                      extract_codeblock/;

# Source filter.
FILTER {
    my $id = qr/\b(?!([ysm]|q[rqxw]?|tr)\b)\w+/;

    # This lexes the Perl source code, replacing quotelike sql//
    # operators with the result of process_sql().
    while($_ !~ /\G$/gc) {
        my $sql;
        my $last_pos = pos();
        if(/\G\s+/gc) {    # whitespace
        }
        elsif(/\G#.*/gc) { # comments
        }
        # sql// operators
        #FIX:should any other quote delimiters be added?
        #FIX:handling of q// v.s. qq//.  Which semantics should
        #    sql// have?
        elsif(/\G\bsql\b\s*(?=[\{\(\[\<\/])/gcs &&
            do {
                my $pos = pos();
                s/\G/ q/; # convert to Perl quote-like
                pos() = $pos;
                $sql = (extract_quotelike())[5];
                #print STDERR "<sql:$sql>";
                if(!$sql) { # restore
                    s/\G q//;
                    pos() = $pos;
                }
                !!$sql;
            }
        )
        {
            #print "[sql:$sql]";
            #print STDERR "[", pos();
            my $pos = pos();
            my $out = &process_sql($sql);
            pos() = $pos;
            #print STDERR "[", pos();
            #print $last_pos, "]";
            substr($_, $last_pos, pos() - $last_pos) = $out;
            pos() = $last_pos + length($out);
        }
        # prevent things like $y = ... = from being interpreted as string.
        elsif(/\G(?<=[\$\@])\w+/gc) {
            #print "[DEBUG:var:$&]";
        }
        elsif(/\G$id/gc) {
            #print "[DEBUG:id:$&]";
        }
        elsif(my $next = (extract_quotelike())[0]) {
            #print "[DEBUG:q:$next]";
        }
        else {
            /\G./gc;
        }
    }
    print STDERR "DEBUG:filter[code=$_]" if $SQL::Interpolate::trace_filter;
};

# Convert the string inside a sql// quote-like operator into
# a list of SQL strings and variable references for interpolation.
sub process_sql
{
    local $_ = shift;

    my @parts;
    my $instr = 0;
    while($_ !~ /\G$/gc) {
        my $tok;
        my $tok_type;
        my $pos_last = pos();
        if(/\G(\s+|\*)/gc) {
            $tok = $1;
            $tok_type = 's';
        }
        elsif(/\GLIMIT\s*/gcs &&
            do {
                $tok = (extract_codeblock($_, undef, undef, '()'))[0];
                #print "<sql:$sql>";
                if($tok) { # nothing
                }
                else { # restore
                    pos() = $pos_last;
                }
                !!$tok;
            }
        ){
            $tok_type = 'limit';
        }
        elsif($tok = (extract_variable($_))[0]) {
            $tok_type = 'v';
        }
        elsif($tok = (extract_codeblock($_, '{['))[0]) {
            $tok_type = 'c';
        }
        else {
            /\G(.)/gc;
            $tok = $1;
            $tok_type = 's';
        }

        if($tok_type eq 's') {
            if($instr) {
                $parts[-1] .= $tok
            }
            else {
                push @parts, $tok
            }
            $instr = 1;
        }
        else {
            $parts[-1] = 'qq[' . $parts[-1] . ']' if $instr;
            $instr = 0;
            if($tok_type eq 'v') {
                push @parts, '\\' . $tok;
            }
            elsif($tok_type eq 'c') {
                push @parts, $tok;
            }
            elsif($tok_type eq 'limit') {
                # note: full package name to not require import
                push @parts, "SQL::Interpolate::Limit->new$tok";
            }
            else { die 'assert'; }
        }

    }
    $parts[-1] = 'qq[' . $parts[-1] . ']' if $instr;
    #use Data::Dumper; die "[DEBUG_SQL_PARTS:" . Dumper(\@parts) . "]";

    my $out = 'SQL::Interpolate::SQL->new(' . join(', ', @parts) . ')';

    return $out;
}

1;

__END__
#
# This module adds source filtering capability to SQL::Interpolate
# so that the sql// quote-like operator may be used.
#
# Copyright (c) 2003-2005 David Manura
# This file is licensed under the same terms as Perl itself.
#
# Note: Sub::Quotelike provides similar functionality to this module,
# but it is not exactly what I need.  Sub::Quotelike allows you to
# replace quote expressions with calls to your own custom function
# that can return itself and expression.  In Sub::Quotelike, the
# return expression is evalutated within the context of the called
# subroutine rather that in the scope of the caller as is typically
# the case with variable interpolation in strings.  Therefore, SQL
# variable interpolation will not work correctly.  Furthermore, the
# current version (0.03) performs fairly simple, and potentially
# error-prone, source filtering.

# We also do not utilize "FILTER_ONLY quotelike" in Filter::Simple
# since its parsing is fairly simplistic and recognizes things like $y
# = ... = as containing a quote (y=...=).
